@@ -86,7 +86,6 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
         auto model = new MultiDayIncidenceModel;
         model->setPeriodLength(periodLength);
         model->setModel(new IncidenceOccurrenceModel);
-        model->model()->setHandleOwnRefresh(false);
         model->model()->setStart(start);
         model->model()->setLength(length);
         model->model()->setFilter(mFilter);
@@ -205,7 +204,6 @@ QVariant InfiniteCalendarViewModel::data(const QModelIndex &idx, int role) const
             m_weekViewModels[startDate]->setPeriodLength(7);
             m_weekViewModels[startDate]->setFilters(HourlyIncidenceModel::NoAllDay | HourlyIncidenceModel::NoMultiDay);
             m_weekViewModels[startDate]->setModel(new IncidenceOccurrenceModel);
-            m_weekViewModels[startDate]->model()->setHandleOwnRefresh(false);
             m_weekViewModels[startDate]->model()->setStart(startDate);
             m_weekViewModels[startDate]->model()->setLength(7);
             m_weekViewModels[startDate]->model()->setFilter(mFilter);
@@ -408,133 +406,23 @@ void InfiniteCalendarViewModel::setCalendar(Akonadi::ETMCalendar *calendar)
     m_insertedIds.clear();
     m_calendar = calendar;
 
-    for (auto model : m_monthViewModels) {
+    for (auto model : std::as_const(m_monthViewModels)) {
         model->model()->setCalendar(calendar);
     }
 
-    for (auto model : m_scheduleViewModels) {
+    for (auto model : std::as_const(m_scheduleViewModels)) {
         model->model()->setCalendar(calendar);
     }
 
-    for (auto model : m_weekViewModels) {
+    for (auto model : std::as_const(m_weekViewModels)) {
         model->model()->setCalendar(calendar);
     }
 
-    for (auto model : m_weekViewMultiDayModels) {
+    for (auto model : std::as_const(m_weekViewMultiDayModels)) {
         model->model()->setCalendar(calendar);
     }
 
-    connect(m_calendar->model(), &QAbstractItemModel::dataChanged, this, &InfiniteCalendarViewModel::handleCalendarDataChanged);
-    connect(m_calendar->model(), &QAbstractItemModel::rowsInserted, this, &InfiniteCalendarViewModel::handleCalendarRowsInserted);
-    connect(m_calendar->model(), &QAbstractItemModel::rowsRemoved, this, &InfiniteCalendarViewModel::handleCalendarRowsRemoved);
-}
-
-void InfiniteCalendarViewModel::checkModels(const QDate &start, const QDate &end, KCalendarCore::Incidence::Ptr incidence)
-{
-    for (auto &model : m_models) {
-        auto modelKeys = model.modelType != TypeWeek ? model.multiDayModels->keys() : model.weekModels->keys();
-
-        for (const auto &modelStartDate : modelKeys) {
-            if (model.affectedStartDates.contains(modelStartDate)) {
-                continue;
-            }
-
-            QDate modelEndDate =
-                model.modelType == TypeSchedule ? modelStartDate.addDays(modelStartDate.daysInMonth()) : modelStartDate.addDays(model.modelLength);
-
-            if (incidence->recurs() && incidence->recurrence()->timesInInterval(modelStartDate.startOfDay(), modelEndDate.endOfDay()).length()) {
-                model.affectedStartDates.append(modelStartDate);
-            } else if (!incidence->recurs()
-                       && (((start <= modelStartDate) && (end >= modelStartDate)) || ((start < modelEndDate) && (end > modelEndDate))
-                           || ((start >= modelStartDate) && (end <= modelEndDate)))) {
-                model.affectedStartDates.append(modelStartDate);
-            }
-        }
-    }
-}
-
-void InfiniteCalendarViewModel::checkCalendarIndex(const QModelIndex &index)
-{
-    const auto item = index.data(Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
-
-    if (item.hasPayload<KCalendarCore::Incidence::Ptr>()) {
-        // If the id is already in the set then we don't need to check anything
-        const auto incidence = item.payload<KCalendarCore::Incidence::Ptr>();
-
-        if (incidence->type() == KCalendarCore::Incidence::TypeTodo) {
-            const auto todo = incidence.staticCast<KCalendarCore::Todo>();
-            const QDate dateStart = todo->dtStart().date();
-            const QDate dateDue = todo->dtDue().date();
-
-            if (!dateStart.isValid() && !dateDue.isValid()) {
-                return;
-            } else if (!dateStart.isValid()) {
-                checkModels(dateDue, dateDue, incidence);
-            }
-
-        } else if (incidence->type() == KCalendarCore::Incidence::TypeEvent) {
-            const auto event = incidence.staticCast<KCalendarCore::Event>();
-            const QDate dateStart = event->dtStart().date();
-            const QDate dateEnd = event->dtEnd().date();
-
-            checkModels(dateStart, dateEnd, incidence);
-        }
-    }
-}
-
-void InfiniteCalendarViewModel::triggerAffectedModelUpdates()
-{
-    for (auto &model : m_models) {
-        if (model.modelType != TypeWeek) {
-            for (const auto &startDate : model.affectedStartDates) {
-                model.multiDayModels->value(startDate)->model()->updateQuery();
-            }
-        } else {
-            for (const auto &startDate : model.affectedStartDates) {
-                model.weekModels->value(startDate)->model()->updateQuery();
-            }
-        }
-    }
-}
-
-void InfiniteCalendarViewModel::handleCalendarDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &)
-{
-    const auto parent = topLeft.parent();
-    int first = topLeft.row();
-    int last = bottomRight.row();
-
-    for (int i = first; i <= last; i++) {
-        auto index = m_calendar->model()->index(i, 0, parent);
-        checkCalendarIndex(index);
-    }
-
-    triggerAffectedModelUpdates();
-}
-
-void InfiniteCalendarViewModel::handleCalendarRowsInserted(const QModelIndex &parent, int first, int last)
-{
-    for (int i = first; i <= last; i++) {
-        const auto index = m_calendar->model()->index(i, 0, parent);
-        const auto item = index.data(Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
-
-        // Sometimes the calendar updates with inserted rows for incidences that we have already checked.
-        // We keep track of which incidence ids have already been checked on prior row insertions, so that
-        // we can skip these ones.
-
-        if (!m_insertedIds.contains(item.id())) {
-            m_insertedIds.insert(item.id());
-            checkCalendarIndex(index);
-        }
-    }
-
-    triggerAffectedModelUpdates();
-}
-
-void InfiniteCalendarViewModel::handleCalendarRowsRemoved(const QModelIndex &, int, int)
-{
-    // We don't know if this is a collection being removed or an incidence being deleted, so be safe
-    // The IncidenceOccurrenceModels also handle this themselves for this reason
-    m_insertedIds.clear();
+    Q_EMIT calendarChanged();
 }
 
 QVariantMap InfiniteCalendarViewModel::filter() const
@@ -545,16 +433,16 @@ QVariantMap InfiniteCalendarViewModel::filter() const
 void InfiniteCalendarViewModel::setFilter(const QVariantMap &filter)
 {
     mFilter = filter;
-    for (auto model : m_monthViewModels) {
+    for (auto model : std::as_const(m_monthViewModels)) {
         model->model()->setFilter(filter);
     }
-    for (auto model : m_scheduleViewModels) {
+    for (auto model : std::as_const(m_scheduleViewModels)) {
         model->model()->setFilter(filter);
     }
-    for (auto model : m_weekViewModels) {
+    for (auto model : std::as_const(m_weekViewModels)) {
         model->model()->setFilter(filter);
     }
-    for (auto model : m_weekViewMultiDayModels) {
+    for (auto model : std::as_const(m_weekViewMultiDayModels)) {
         model->model()->setFilter(filter);
     }
     Q_EMIT filterChanged();
